<?php

namespace App\Command;

use App\Entity\User;
use App\Repository\UserRepository;
use Symfony\Component\Console\Attribute\AsCommand;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;

#[AsCommand(
    name: 'blog:user:create',
    description: 'Créer le premier utilisateur',
)]
class BlogUserCreateCommand extends Command
{
    public function __construct(
        protected readonly UserRepository $userRepository,
        protected UserPasswordHasherInterface $passwordHasher
    )
    {
        parent::__construct();
    }

    protected function configure(): void
    {
        
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $io = new SymfonyStyle($input, $output);
        
        $user = new User();

        $user->setUsername($io->ask('Username'));
        $user->setPseudo($io->ask('Pseudo'));
        $user->setEmail($io->ask('Email'));
        do {
            $password = $io->askHidden('Mot de passe');
            $confirm = $io->askHidden('Confirmez le mot de passe');
        } while ($password !== $confirm);
        
        $user->setPassword($this->passwordHasher->hashPassword($user, $password));

        if ($io->confirm("Est-ce un administrateur", true)) {
            $user->setRoles(['ROLE_ADMIN']);
        } else if ($io->confirm("Est-il autorisé à modifier la page contacts (Contacts + auteur)")) {
            $user->setRoles(['ROLE_CONTACT']);
        } else {
            $user->setRoles(['ROLE_AUTHOR']);
        }

        $this->userRepository->add($user);

        $io->success('Utilisateur créé');

        return Command::SUCCESS;
    }
}
