<?php

namespace App\Form;

use App\Entity\Article;
use App\Entity\User;
use App\Service\BaseUrl;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;

class ArticleType extends AbstractType
{
    public function __construct(protected BaseUrl $baseUrl)
    {
        
    }

    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $builder
            ->add('title', TextType::class, [
                'attr'  => [
                    'data-slugger-target'  => 'title'
                ]
            ])
            ->add('slug', TextType::class, [
                'attr'  => [
                    //'readonly'  => true,
                    'data-slugger-target'  => 'slug',
                    'data-url' => $this->baseUrl->getBasePath()
                ],
            ])
            ->add('content', TextareaType::class, [
                'required'  => false,
                'attr'  =>  [
                    'data-controller'   => 'easymde',
                    'data-url' => $this->baseUrl->getBasePath()
                ]
            ])
            ->add('publicationDate', null, [
                'widget' => 'single_text',
            ])
            ->add('author', EntityType::class, [
                'class' => User::class,
                'choice_label' => 'userIdentifier',
            ])
            ->add('publicationChoice', ChoiceType::class, [
                'mapped' => false,
                'choices' => $options['transitions'],
                'required' => false,
                'choice_value'  => null
            ])
        ;
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'data_class' => Article::class,
        ]);

        $resolver->setRequired('transitions');
    }
}
