<?php

namespace App\Security\Voter;

use App\Entity\Article;
use App\Entity\User;
use Symfony\Bundle\SecurityBundle\Security;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Symfony\Component\Security\Core\Authorization\Voter\Voter;

final class ArticleVoter extends Voter
{
    public const EDIT = 'edit';
    public const SHOW = 'show';
    public const VIEW = 'view';
    public const PUBLISH = 'publish';

    public function __construct(
        private readonly Security $security,
    )
    {
        
    }

    protected function supports(string $attribute, mixed $subject): bool
    {
        // replace with your own logic
        // https://symfony.com/doc/current/security/voters.html
        return in_array($attribute, [self::EDIT, self::VIEW, self::PUBLISH, self::SHOW])
            && $subject instanceof \App\Entity\Article;
    }

    protected function voteOnAttribute(string $attribute, mixed $subject, TokenInterface $token): bool
    {
        $user = $token->getUser();

        // // if the user is anonymous, do not grant access
        // if (!$user instanceof User) {
        //     return false;
        // }

        $article = $subject;

        return match($attribute) {
            self::VIEW => $this->canView($article, $user),
            self::SHOW => $this->canShow($article, $user),
            self::EDIT => $this->canEdit($article, $user),
            self::PUBLISH => $this->canPublish($article, $user),
            default => throw new \LogicException('This code should not be reached!')
        };
    }

    private function canView(Article $article, ?User $user): bool
    {
        return $article->getState() === 'published' || $this->canEdit($article, $user);
    }

    private function canShow(Article $article, ?User $user): bool
    {
        return $this->canEdit($article, $user) || $this->security->isGranted('ROLE_MODERATOR');
    }

    private function canEdit(Article $article, ?User $user): bool
    {
        if(!$user instanceof User) {
            return false;
        }
 
        return $user === $article->getAuthor() || $this->security->isGranted('ROLE_ADMIN');
    }

    private function canPublish(Article $article, ?User $user) :bool
    {
        if ($this->canEdit($article, $user) && $article->getState() !== 'reviewed') {
            return true;
        }

        return $this->security->isGranted('ROLE_MODERATOR');
    }
}
