<?php

namespace App\Service;

use App\Markdown\Extension\ColorExtension;
use App\Markdown\Extension\IntroExtension;
use Embed\Embed;
use League\CommonMark\Environment\Environment;
use League\CommonMark\Extension\Autolink\AutolinkExtension;
use League\CommonMark\Extension\CommonMark\CommonMarkCoreExtension;
use League\CommonMark\Extension\Embed\Bridge\OscaroteroEmbedAdapter;
use League\CommonMark\Extension\Embed\EmbedExtension;
use League\CommonMark\Extension\ExternalLink\ExternalLinkExtension;
use League\CommonMark\Extension\Table\TableExtension;
use League\CommonMark\MarkdownConverter;

class MarkdownParser
{
    protected MarkdownConverter $converter;

    public function __construct()
    {
        $embedLibrary = new Embed();
        $embedLibrary->setSettings([
            'oembed:query_parameters' => [
                'maxwidth' => 500,
                'maxheight' => 400,
            ],
        ]);
        $config = [
            'embed' => [
                'adapter' => new OscaroteroEmbedAdapter($embedLibrary), // See the "Adapter" documentation below
                'allowed_domains' => ['youtu.be', 'youtube.com', 'twitter.com', 'github.com'],
                'fallback' => 'link',
            ],
            'autolink' => [
                'allowed_protocols' => ['https'], // defaults to ['https', 'http', 'ftp']
                'default_protocol' => 'https'
            ],
            'external_link' => [
                'internal_hosts' => 'https://opinionsurlereel.fdlibre.eu',
                'open_in_new_window' => true,
                'html_class' => 'external-link',
                'nofollow' => '',
                'noopener' => 'external',
                'noreferrer' => 'external',
            ],
            'table' => [
                'wrap' => [
                    'enabled' => false,
                    'tag' => 'div',
                    'attributes' => [],
                ],
                'alignment_attributes' => [
                    'left'   => ['align' => 'left'],
                    'center' => ['align' => 'center'],
                    'right'  => ['align' => 'right'],
                ],
            ],
        ];

        $environment = new Environment($config);
        $environment->addExtension(new CommonMarkCoreExtension());
        $environment->addExtension(new ExternalLinkExtension());
        $environment->addExtension(new AutolinkExtension());
        $environment->addExtension(new TableExtension());
        $environment->addExtension(new EmbedExtension());
        $environment->addExtension(new ColorExtension());
        $environment->addExtension(new IntroExtension());

        $this->converter = new MarkdownConverter($environment);
    }

    public function convertToHtml(string $markdown): string
    {
        $html = $this->converter->convert($markdown);

        return $html;
    }
}
